# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cmsplugin_filer_link2', '0004_auto_20170306_1751'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='linkhealthstate',
            options={'verbose_name': 'Link Health State', 'verbose_name_plural': 'Link Health States'},
        ),
        migrations.AlterField(
            model_name='linkhealthstate',
            name='detected',
            field=models.DateTimeField(help_text='Date and time when the faulty link state was detected.', verbose_name='Detected on', auto_now=True),
        ),
        migrations.AlterField(
            model_name='linkhealthstate',
            name='link',
            field=models.OneToOneField(related_name='linkhealth', verbose_name='Link', to='cmsplugin_filer_link2.FilerLink2Plugin'),
        ),
        migrations.AlterField(
            model_name='linkhealthstate',
            name='state',
            field=models.CharField(max_length=3, verbose_name='State', choices=[('3xx', 'Redirected'), ('4xx', 'Not reachable'), ('5xx', 'Server error'), ('bad', 'Bad configured')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cmsplugin_filer_link2', '0003_linkhealthstate'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filerlink2plugin',
            name='url',
            field=models.CharField(help_text='The url must specify the protocol, e.g. https://DOMAIN', max_length=2000, null=True, verbose_name='url', blank=True),
        ),
        migrations.AlterField(
            model_name='linkhealthstate',
            name='state',
            field=models.CharField(max_length=3, choices=[('3xx', 'Redirected'), ('4xx', 'Not reachable'), ('5xx', 'Server error'), ('bad', 'Bad configured')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.core.urlresolvers import NoReverseMatch
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from cms.models import CMSPlugin
from cms.models.fields import PageField

from filer.fields.file import FilerFileField
from filer.utils.compatibility import python_2_unicode_compatible

from djangocms_attributes_field.fields import AttributesField


DEFULT_LINK_STYLES = (
    (" ", "Default"),
)

LINK_STYLES = getattr(settings, "FILER_LINK_STYLES", DEFULT_LINK_STYLES)

EXCLUDED_KEYS = ['class', 'href', 'target', ]


@python_2_unicode_compatible
class FilerLink2Plugin(CMSPlugin):
    name = models.CharField(_('name'), max_length=255)
    url = models.CharField(_('url'), blank=True, null=True, max_length=2000,
                           help_text=_('The url must specify the protocol, e.g. https://DOMAIN.tld'))
    page_link = PageField(
        verbose_name=_('page'),
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
    )
    persistent_page_link = models.CharField(_('internal url'), blank=True, null=True, max_length=2000)
    mailto = models.EmailField(_('mailto'), blank=True, null=True, max_length=254)
    link_style = models.CharField(_('link style'), max_length=255,
                                  choices=LINK_STYLES, default=LINK_STYLES[0][0])
    new_window = models.BooleanField(_('new window?'), default=False,
                                     help_text=_('Do you want this link to open a new window?'))
    file = FilerFileField(blank=True, null=True, on_delete=models.SET_NULL)
    link_attributes = AttributesField(excluded_keys=EXCLUDED_KEYS, blank=True,
                                      help_text=_('Optional. Adds HTML attributes to the rendered link.'))
    cmsplugin_ptr = models.OneToOneField(
        to=CMSPlugin,
        related_name='%(app_label)s_%(class)s',
        parent_link=True,
    )

    def __str__(self):
        return self.name

    def clean(self):
        super(FilerLink2Plugin, self).clean()
        configured_destinations = [d for d in
                                   ('url', 'page_link', 'mailto', 'file')
                                   if getattr(self, d) is not None and getattr(self, d) != '']
        if len(configured_destinations) == 0:
            raise ValidationError(_('Please choose a destination'))
        elif len(configured_destinations) > 1:
            raise ValidationError(
                _('Please only choose one destination! You set: {}'.format(', '.join(configured_destinations))))

    def save(self, *args, **kwargs):
        super(FilerLink2Plugin, self).save(*args, **kwargs)
        # delete link health state
        LinkHealthState.objects.filter(link=self).delete()

    def get_link(self):
        if self.file:
            link = self.file.url
        elif self.mailto:
            link = 'mailto:{}'.format(_(self.mailto))
        elif self.url:
            link = _(self.url)
        elif self.page_link:
            try:
                link = self.page_link.get_absolute_url()
            except NoReverseMatch:
                # if this internal link doesn't work anymore, we mark it not reachable
                self.set_linkstate(LinkHealthState.NOT_REACHABLE)
                # return old internal link and send user to 404
                link = self.persistent_page_link
            else:
                # check if the target page has been moved or renamed and update accordingly
                if link != self.persistent_page_link:
                    self.persistent_page_link = link
                    self.save()
        elif self.persistent_page_link:
            # happens when this link instance pointed to a removed page
            self.set_linkstate(LinkHealthState.NOT_REACHABLE)
            link = self.persistent_page_link
        else:
            link = ''
        return link or ''

    def set_linkstate(self, state):
        if state is None:
            LinkHealthState.objects.filter(link=self).delete()
        else:
            LinkHealthState.objects.update_or_create(link=self, defaults={'state': state})

    def get_linkstate(self):
        try:
            return self.linkhealth.state
        except ObjectDoesNotExist:
            return None

    @property
    def active_destination(self):
        """ The active destination determines which destination tab should be set to active. If the field is not set
        yet, we return None
        :return: field_name: str
        """
        configured_destinations = [d for d in
                                   ('url', 'page_link', 'mailto', 'file')
                                   if getattr(self, d) is not None and getattr(self, d) != '']
        if len(configured_destinations) == 0:
            return None
        return configured_destinations[0]


@python_2_unicode_compatible
class LinkHealthState(models.Model):

    NOT_REACHABLE = '4xx'
    REDIRECT = '3xx'
    SERVER_ERROR = '5xx'
    BAD_CONFIGURED = 'bad'

    LINK_STATES = (
        (REDIRECT, _('Redirected')),
        (NOT_REACHABLE, _('Not reachable')),
        (SERVER_ERROR, _('Server error')),
        (BAD_CONFIGURED, _('Bad configured')),
    )

    link = models.OneToOneField(FilerLink2Plugin, unique=True, related_name='linkhealth',
                                verbose_name=_('Link name'))
    state = models.CharField(max_length=3, choices=LINK_STATES, verbose_name=_('State'))
    detected = models.DateTimeField(auto_now=True, verbose_name=_('Detected on'),
                                    help_text=_('Date and time when the faulty link state was detected.'))

    def __str__(self):
        return _(u'Link state for: {}').format(self.link.name)

    class Meta:
        verbose_name = _('Link Health State')
        verbose_name_plural = _('Link Health States')



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.safestring import mark_safe
from django.utils.translation import activate
from django.utils.translation import ugettext as _

from .models import LinkHealthState


class LinkStateAdmin(admin.ModelAdmin):
    list_display = ('link_name', 'link_to', 'state', 'on_page', 'detected')
    list_filter = ('state',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def link_name(self, obj):
        return obj.link
    link_name.allow_tags = True
    link_name.short_description = _('Link name')

    def on_page(self, obj):
        activate(obj.link.language)
        return mark_safe('<a href="{link}" >{link}</a>'.format(link=obj.link.page.get_absolute_url()))
    on_page.allow_tags = True
    on_page.short_description = _('On page')

    def link_to(self, obj):
        if obj.state != LinkHealthState.BAD_CONFIGURED:
            activate(obj.link.language)
            return mark_safe('<a href="{link}" >{link}</a>'.format(link=obj.link.get_link()))
    link_to.allow_tags = True
    link_to.short_description = _('Links to')


admin.site.register(LinkHealthState, LinkStateAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.templatetags.static import static
from django.utils.translation import ugettext as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from .forms import FilerLink2Form
from .models import FilerLink2Plugin as FilerLinkPluginModel


class FilerLink2Plugin(CMSPluginBase):
    form = FilerLink2Form
    model = FilerLinkPluginModel
    module = 'Filer'
    name = _('Link')
    raw_id_fields = ('page_link', )
    render_template = "cmsplugin_filer_link/link.html"
    change_form_template = 'cmsplugin_filer_link/change_form.html'
    text_enabled = True
    fieldsets = (
        (None, {
            'fields': [
                'name',
            ]
        }),
        (None, {
            'classes': ['link2-destination', ],
            'fields': [
                'url',
                'page_link',
                'mailto',
                'file',
            ],
        }),
        (None, {
            'fields': [
                'new_window',
                'link_style',
            ]
        }),
        (_('Advanced'), {
            'classes': ['collapse', ],
            'fields': [
                'link_attributes',
            ]
        })
    )

    def render(self, context, instance, placeholder):
        context = super(FilerLink2Plugin, self).render(context, instance, placeholder)
        link = instance.get_link()
        context.update({
            'link': link,
            'style': instance.link_style,
            'name': instance.name,
            'new_window': instance.new_window,
        })
        try:
            # check if we are in edit mode, so we show link health
            if context['request'].toolbar.edit_mode:
                state = instance.get_linkstate()
                if state:
                    context.update({
                        'link_state': state,
                    })
        except (KeyError, AttributeError):
            pass
        return context

    def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
        # The active destination determines which destination tab should be set to active. If the field is not set
        # yet, we make the first tab (url) active
        context.update({
            'active_destination': 'url' if add is True or obj.active_destination is None else obj.active_destination
        })
        return super(FilerLink2Plugin, self).render_change_form(request, context, add, change, form_url, obj)

    def icon_src(self, instance):
        return static("cms/img/icons/plugins/link.png")


plugin_pool.register_plugin(FilerLink2Plugin)



from cms.models import CMSPlugin
from cmsplugin_filer_link.models import FilerLinkPlugin
from django.core.management.base import BaseCommand
from django.db import transaction

from cmsplugin_filer_link2.models import FilerLink2Plugin


class Command(BaseCommand):
    help = 'Migrate all FilerLinkPlugin to FilerLink2Plugins'

    def handle(self, *args, **options):
        old_links = FilerLinkPlugin.objects.all()
        self.stdout.write('Migrating {num} FilerLinkPlugin objects to FilerLink2Plugins'.format(num=old_links.count()))

        with transaction.atomic():
            for old_link in old_links:
                cmsplugin = CMSPlugin.objects.get(id=old_link.cmsplugin_ptr_id)
                FilerLinkPlugin.objects.filter(id=old_link.id).delete()

                link = FilerLink2Plugin(cmsplugin_ptr=cmsplugin)
                link.__dict__.update(old_link.__dict__)
                link.plugin_type = 'FilerLink2Plugin'
                link.save()
                link.parent.numchild += 1
                link.parent.save()



import requests

from django.core.management.base import BaseCommand
from django.core.urlresolvers import NoReverseMatch

from django.utils.translation import activate
from requests.exceptions import ConnectionError, MissingSchema

from cmsplugin_filer_link2.models import FilerLink2Plugin, LinkHealthState


class Command(BaseCommand):
    help = 'Check all links for the availability of their destination'

    def check_with_request(self, url):
        try:
            r = requests.get(url)
        except ConnectionError:
            return LinkHealthState.SERVER_ERROR
        except MissingSchema:
            return LinkHealthState.BAD_CONFIGURED

        return {
            # we are only interested in bad status codes
            '3': LinkHealthState.REDIRECT,
            '4': LinkHealthState.NOT_REACHABLE,
            '5': LinkHealthState.SERVER_ERROR
        }.get(str(r.status_code)[0])

    def handle(self, *args, **options):
        all_links = FilerLink2Plugin.objects.all()
        self.stdout.write('Checking {num} link-instances'.format(num=all_links.count()))

        for link in all_links:
            status = None
            if link.file:
                status = self.check_with_request(link.file.url)
            elif link.url:
                status = self.check_with_request(link.url)
            elif link.page_link:
                try:
                    # see if we can resolve the page this link points to
                    activate(link.language)
                    link.page_link.get_absolute_url()
                except NoReverseMatch:
                    status = LinkHealthState.NOT_REACHABLE
            link.set_linkstate(status)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cmsplugin_filer_link2', '0002_filerlink2plugin_persistent_page_link'),
    ]

    operations = [
        migrations.CreateModel(
            name='LinkHealthState',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('state', models.CharField(max_length=3, choices=[('3xx', 'Redirected'), ('4xx', 'Not reachable'), ('5xx', 'Server error')])),
                ('detected', models.DateTimeField(auto_now=True)),
                ('link', models.OneToOneField(related_name='linkhealth', to='cmsplugin_filer_link2.FilerLink2Plugin')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cmsplugin_filer_link2', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='filerlink2plugin',
            name='persistent_page_link',
            field=models.CharField(max_length=2000, null=True, verbose_name='internal url', blank=True),
        ),
    ]
